@@ -35,6 +35,7 @@
 #include "bz-global-net.h"
 #include "bz-io.h"
 #include "bz-release.h"
+#include "bz-repository.h"
 #include "bz-serializable.h"
 #include "bz-url.h"
 #include "bz-util.h"
@@ -72,6 +73,7 @@ typedef struct
 {
   gint     hold;
   gboolean installed;
+  gboolean searchable;
 
   guint             kinds;
   GListModel       *addons;
@@ -140,6 +142,7 @@ enum
 
   PROP_HOLDING,
   PROP_INSTALLED,
+  PROP_SEARCHABLE,
   PROP_KINDS,
   PROP_ADDONS,
   PROP_ID,
@@ -294,6 +297,9 @@ bz_entry_get_property (GObject    *object,
     case PROP_INSTALLED:
       g_value_set_boolean (value, priv->installed);
       break;
+    case PROP_SEARCHABLE:
+      g_value_set_boolean (value, priv->searchable);
+      break;
     case PROP_ADDONS:
       g_value_set_object (value, priv->addons);
       break;
@@ -480,6 +486,9 @@ bz_entry_set_property (GObject      *object,
     case PROP_INSTALLED:
       priv->installed = g_value_get_boolean (value);
       break;
+    case PROP_SEARCHABLE:
+      priv->searchable = g_value_get_boolean (value);
+      break;
     case PROP_ADDONS:
       g_clear_object (&priv->addons);
       priv->addons = g_value_dup_object (value);
@@ -735,6 +744,13 @@ bz_entry_class_init (BzEntryClass *klass)
           NULL, NULL, FALSE,
           G_PARAM_READWRITE);
 
+  props[PROP_SEARCHABLE] =
+      g_param_spec_boolean (
+          "searchable",
+          NULL, NULL,
+          TRUE,
+          G_PARAM_READWRITE);
+
   props[PROP_ADDONS] =
       g_param_spec_object (
           "addons",
@@ -1098,6 +1114,7 @@ bz_entry_init (BzEntry *self)
   BzEntryPrivate *priv = bz_entry_get_instance_private (self);
 
   priv->hold            = 0;
+  priv->searchable      = TRUE;
   priv->favorites_count = -1;
 }
 
@@ -1110,6 +1127,7 @@ bz_entry_real_serialize (BzSerializable  *serializable,
 
   g_variant_builder_add (builder, "{sv}", "installed", g_variant_new_boolean (priv->installed));
   g_variant_builder_add (builder, "{sv}", "kinds", g_variant_new_uint32 (priv->kinds));
+  g_variant_builder_add (builder, "{sv}", "searchable", g_variant_new_boolean (priv->searchable));
   if (priv->addons != NULL)
     {
       guint n_items = 0;
@@ -1477,6 +1495,8 @@ bz_entry_real_deserialize (BzSerializable *serializable,
         priv->installed = g_variant_get_boolean (value);
       else if (g_strcmp0 (key, "kinds") == 0)
         priv->kinds = g_variant_get_uint32 (value);
+      else if (g_strcmp0 (key, "searchable") == 0)
+        priv->searchable = g_variant_get_boolean (value);
       else if (g_strcmp0 (key, "addons") == 0)
         {
           g_autoptr (GListStore) store        = NULL;
@@ -1867,6 +1887,17 @@ bz_entry_is_of_kinds (BzEntry *self,
   return (priv->kinds & kinds) == kinds;
 }
 
+gboolean
+bz_entry_is_searchable (BzEntry *self)
+{
+  BzEntryPrivate *priv = NULL;
+
+  g_return_val_if_fail (BZ_IS_ENTRY (self), TRUE);
+  priv = bz_entry_get_instance_private (self);
+
+  return priv->searchable;
+}
+
 void
 bz_entry_append_addon (BzEntry    *self,
                        const char *id)
@@ -2124,6 +2155,46 @@ bz_entry_get_forge_url (BzEntry *self)
   return priv->forge_url;
 }
 
+BzRepository *
+bz_entry_get_repository (BzEntry    *self,
+                         GListModel *repos)
+{
+  BzEntryPrivate *priv    = NULL;
+  guint           n_repos = 0;
+  g_auto (GStrv) parts    = NULL;
+  const char *scope       = NULL;
+  const char *repo_name   = NULL;
+  gboolean    is_user     = FALSE;
+
+  priv = bz_entry_get_instance_private (self);
+
+  if (priv->unique_id == NULL)
+    return NULL;
+
+  parts = g_strsplit (priv->unique_id, "::", -1);
+  if (g_strv_length (parts) < 3)
+    return NULL;
+
+  scope     = parts[0];
+  repo_name = parts[1];
+
+  is_user = g_strcmp0 (scope, "FLATPAK-USER") == 0;
+
+  n_repos = g_list_model_get_n_items (repos);
+  for (guint i = 0; i < n_repos; i++)
+    {
+      g_autoptr (BzRepository) repo = g_list_model_get_item (repos, i);
+      const char *name              = bz_repository_get_name (repo);
+      gboolean    repo_is_user      = bz_repository_get_is_user (repo);
+
+      if (repo_is_user == is_user &&
+          g_strcmp0 (name, repo_name) == 0)
+        return g_object_ref (repo);
+    }
+
+  return NULL;
+}
+
 gboolean
 bz_entry_get_is_foss (BzEntry *self)
 {
@@ -2384,10 +2455,15 @@ query_flathub (BzEntry *self,
   BzEntryPrivate *priv              = NULL;
   g_autoptr (QueryFlathubData) data = NULL;
   g_autoptr (DexFuture) future      = NULL;
+  gboolean is_download_stat         = FALSE;
 
   priv = bz_entry_get_instance_private (self);
 
-  if (!priv->is_flathub)
+  is_download_stat = (prop == PROP_DOWNLOAD_STATS ||
+                      prop == PROP_DOWNLOAD_STATS_PER_COUNTRY ||
+                      prop == PROP_TOTAL_DOWNLOADS);
+
+  if (!is_download_stat && !priv->is_flathub)
     return;
   if (priv->id == NULL)
     return;
