@@ -32,6 +32,7 @@
 #include "bz-flatpak-private.h"
 #include "bz-global-net.h"
 #include "bz-io.h"
+#include "bz-repository.h"
 #include "bz-util.h"
 
 /* clang-format off */
@@ -133,6 +134,19 @@ retrieve_installs_fiber (GatherRefsData *data);
 static DexFuture *
 retrieve_updates_fiber (GatherRefsData *data);
 
+BZ_DEFINE_DATA (
+    list_repos,
+    ListRepos,
+    {
+      GWeakRef     *self;
+      GCancellable *cancellable;
+    },
+    BZ_RELEASE_DATA (self, bz_weak_release);
+    BZ_RELEASE_DATA (cancellable, g_object_unref));
+
+static DexFuture *
+list_repositories_fiber (ListReposData *data);
+
 BZ_DEFINE_DATA (
     retrieve_refs_for_remote,
     RetrieveRefsForRemote,
@@ -292,6 +306,24 @@ cmp_rref (FlatpakRemoteRef *a,
           FlatpakRemoteRef *b,
           GHashTable       *hash);
 
+static AsComponent *
+parse_component_for_node (XbNode  *node,
+                          GError **error);
+
+static GBytes *
+decompress_appstream_gz (GBytes       *appstream_gz,
+                         GCancellable *cancellable,
+                         GError      **error);
+
+static XbSilo *
+build_silo (XbBuilderSource *source,
+            GCancellable    *cancellable,
+            GError         **error);
+
+static AsComponent *
+extract_first_component_for_silo (XbSilo  *silo,
+                                  GError **error);
+
 static void
 bz_flatpak_instance_dispose (GObject *object)
 {
@@ -426,6 +458,25 @@ bz_flatpak_instance_retrieve_update_ids (BzBackend    *backend,
       gather_refs_data_unref);
 }
 
+static DexFuture *
+bz_flatpak_instance_list_repositories (BzBackend    *backend,
+                                       GCancellable *cancellable)
+{
+  BzFlatpakInstance *self        = BZ_FLATPAK_INSTANCE (backend);
+  g_autoptr (ListReposData) data = NULL;
+
+  data              = list_repos_data_new ();
+  data->self        = bz_track_weak (self);
+  data->cancellable = bz_object_maybe_ref (cancellable);
+
+  return dex_scheduler_spawn (
+      self->scheduler,
+      bz_get_dex_stack_size (),
+      (DexFiberFunc) list_repositories_fiber,
+      list_repos_data_ref (data),
+      list_repos_data_unref);
+}
+
 static DexFuture *
 bz_flatpak_instance_schedule_transaction (BzBackend    *backend,
                                           BzEntry     **installs,
@@ -497,6 +548,7 @@ backend_iface_init (BzBackendInterface *iface)
   iface->retrieve_remote_entries     = bz_flatpak_instance_retrieve_remote_refs;
   iface->retrieve_install_ids        = bz_flatpak_instance_retrieve_install_ids;
   iface->retrieve_update_ids         = bz_flatpak_instance_retrieve_update_ids;
+  iface->list_repositories           = bz_flatpak_instance_list_repositories;
   iface->schedule_transaction        = bz_flatpak_instance_schedule_transaction;
 }
 
@@ -775,14 +827,19 @@ ensure_flathub_fiber (EnsureFlathubData *data)
 static DexFuture *
 load_local_ref_fiber (LoadLocalRefData *data)
 {
-  // GCancellable      *cancellable    = data->cancellable;
-  // BzFlatpakInstance *instance       = data->instance;
-  GFile *file                       = data->file;
-  g_autoptr (GError) local_error    = NULL;
-  g_autofree char *uri              = NULL;
-  g_autofree char *path             = NULL;
-  g_autoptr (FlatpakBundleRef) bref = NULL;
-  g_autoptr (BzFlatpakEntry) entry  = NULL;
+  GFile *file                                = data->file;
+  g_autoptr (GError) local_error             = NULL;
+  g_autofree char *uri                       = NULL;
+  g_autofree char *path                      = NULL;
+  g_autoptr (FlatpakBundleRef) bref          = NULL;
+  g_autoptr (BzFlatpakEntry) entry           = NULL;
+  g_autoptr (GBytes) appstream_gz            = NULL;
+  gboolean result                            = FALSE;
+  g_autoptr (AsComponent) component          = NULL;
+  g_autoptr (GBytes) appstream               = NULL;
+  g_autoptr (GInputStream) stream_gz         = NULL;
+  g_autoptr (GInputStream) stream_data       = NULL;
+  g_autoptr (GZlibDecompressor) decompressor = NULL;
 
   uri  = g_file_get_uri (file);
   path = g_file_get_path (file);
@@ -793,7 +850,6 @@ load_local_ref_fiber (LoadLocalRefData *data)
     {
       const char *resolved_uri      = NULL;
       g_autoptr (GKeyFile) key_file = g_key_file_new ();
-      gboolean         result       = FALSE;
       g_autofree char *name         = NULL;
 
       if (g_str_has_prefix (uri, "flatpak+https"))
@@ -859,11 +915,54 @@ load_local_ref_fiber (LoadLocalRefData *data)
         path,
         local_error->message);
 
+  appstream_gz = flatpak_bundle_ref_get_appstream (bref);
+  if (appstream_gz != NULL)
+    {
+      g_autoptr (XbBuilderSource) source = NULL;
+      g_autoptr (XbSilo) silo            = NULL;
+
+      appstream = decompress_appstream_gz (appstream_gz, NULL, &local_error);
+      if (appstream == NULL)
+        {
+          g_warning ("Failed to decompress AppStream data: %s", local_error->message);
+          g_clear_error (&local_error);
+        }
+      else
+        {
+          source = xb_builder_source_new ();
+          if (!xb_builder_source_load_bytes (source, appstream,
+                                             XB_BUILDER_SOURCE_FLAG_LITERAL_TEXT,
+                                             &local_error))
+            {
+              g_warning ("Failed to load AppStream bytes into xmlb: %s", local_error->message);
+              g_clear_error (&local_error);
+            }
+          else
+            {
+              silo = build_silo (source, NULL, &local_error);
+              if (silo == NULL)
+                {
+                  g_warning ("Failed to compile xmlb silo: %s", local_error->message);
+                  g_clear_error (&local_error);
+                }
+              else
+                {
+                  component = extract_first_component_for_silo (silo, &local_error);
+                  if (component == NULL && local_error != NULL)
+                    {
+                      g_warning ("Failed to parse component: %s", local_error->message);
+                      g_clear_error (&local_error);
+                    }
+                }
+            }
+        }
+    }
+
   entry = bz_flatpak_entry_new_for_ref (
       FLATPAK_REF (bref),
       NULL,
       FALSE,
-      NULL,
+      component,
       NULL,
       &local_error);
   if (entry == NULL)
@@ -949,13 +1048,6 @@ retrieve_remote_refs_fiber (GatherRefsData *data)
 
       name = flatpak_remote_get_name (remote);
 
-      if (flatpak_remote_get_disabled (remote) ||
-          flatpak_remote_get_noenumerate (remote))
-        {
-          g_debug ("Skipping remote %s", name);
-          continue;
-        }
-
       if (strstr (name, "fedora") != NULL)
         {
           g_debug ("Skipping remote %s", name);
@@ -1031,35 +1123,29 @@ gather_refs_update_progress (const char     *status,
 }
 
 static DexFuture *
-retrieve_refs_for_remote_fiber (RetrieveRefsForRemoteData *data)
+retrieve_refs_for_enumerable_remote (RetrieveRefsForRemoteData *data,
+                                     const char                *remote_name,
+                                     FlatpakInstallation       *installation,
+                                     FlatpakRemote             *remote)
 {
   g_autoptr (BzFlatpakInstance) self    = NULL;
-  GCancellable        *cancellable      = data->parent->cancellable;
-  FlatpakInstallation *installation     = data->installation;
-  FlatpakRemote       *remote           = data->remote;
+  GCancellable *cancellable             = data->parent->cancellable;
   g_autoptr (GError) local_error        = NULL;
-  g_autoptr (DexFuture) error_future    = NULL;
-  const char *remote_name               = NULL;
-  gboolean    result                    = FALSE;
+  gboolean result                       = FALSE;
   g_autoptr (GFile) appstream_dir       = NULL;
   g_autofree char *appstream_dir_path   = NULL;
   g_autofree char *appstream_xml_path   = NULL;
   g_autoptr (GFile) appstream_xml       = NULL;
   g_autoptr (XbBuilderSource) source    = NULL;
-  g_autoptr (XbBuilder) builder         = NULL;
-  const gchar *const *locales           = NULL;
   g_autoptr (XbSilo) silo               = NULL;
   g_autoptr (XbNode) root               = NULL;
   g_autoptr (GPtrArray) children        = NULL;
-  g_autoptr (AsMetadata) metadata       = NULL;
-  AsComponentBox *components            = NULL;
   g_autoptr (GHashTable) component_hash = NULL;
-  g_autoptr (GdkPaintable) remote_icon  = NULL;
   g_autoptr (GPtrArray) refs            = NULL;
 
   bz_weak_get_or_return_reject (self, data->parent->self);
 
-  remote_name = flatpak_remote_get_name (remote);
+  g_debug ("Remote '%s' is enumerable, listing all remote refs", remote_name);
 
   result = flatpak_installation_update_remote_sync (
       installation,
@@ -1131,23 +1217,7 @@ retrieve_refs_for_remote_fiber (RetrieveRefsForRemoteData *data)
         remote_name,
         local_error->message);
 
-  builder = xb_builder_new ();
-  locales = g_get_language_names ();
-  for (guint i = 0; locales[i] != NULL; i++)
-    xb_builder_add_locale (builder, locales[i]);
-  xb_builder_import_source (builder, source);
-
-  silo = xb_builder_compile (
-      builder,
-
-      /* This was causing issues */
-      // // fallback for locales should be handled by AppStream as_component_get_name
-      // XB_BUILDER_COMPILE_FLAG_NONE,
-
-      /* This seems to work better */
-      XB_BUILDER_COMPILE_FLAG_NATIVE_LANGS,
-      cancellable,
-      &local_error);
+  silo = build_silo (source, cancellable, &local_error);
 
 #ifdef __GLIBC__
   /* From gnome-software/plugins/core/gs-plugin-appstream.c
@@ -1170,51 +1240,31 @@ retrieve_refs_for_remote_fiber (RetrieveRefsForRemoteData *data)
 
   root     = xb_silo_get_root (silo);
   children = xb_node_get_children (root);
-  metadata = as_metadata_new ();
+
+  component_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
   for (guint i = 0; i < children->len; i++)
     {
-      XbNode          *component_node = NULL;
-      g_autofree char *component_xml  = NULL;
+      XbNode      *component_node = NULL;
+      AsComponent *component      = NULL;
+      const char  *id             = NULL;
 
       component_node = g_ptr_array_index (children, i);
+      component      = parse_component_for_node (component_node, &local_error);
 
-      component_xml = xb_node_export (
-          component_node, XB_NODE_EXPORT_FLAG_NONE, &local_error);
-      if (component_xml == NULL)
-        SEND_AND_RETURN_ERROR (
-            self, TRUE,
-            BZ_FLATPAK_ERROR_IO_MISBEHAVIOR,
-            "Failed to export plain xml from appstream bundle silo "
-            "originating from download at path %s for remote '%s': %s",
-            appstream_xml_path,
-            remote_name,
-            local_error->message);
-
-      result = as_metadata_parse_data (
-          metadata, component_xml, -1,
-          AS_FORMAT_KIND_XML, &local_error);
-      if (!result)
-        SEND_AND_RETURN_ERROR (
-            self, TRUE,
-            BZ_FLATPAK_ERROR_APPSTREAM_FAILURE,
-            "Failed to create appstream metadata from appstream bundle silo "
-            "originating from download at path %s for remote '%s': %s",
-            appstream_xml_path,
-            remote_name,
-            local_error->message);
-    }
-
-  components     = as_metadata_get_components (metadata);
-  component_hash = g_hash_table_new (g_str_hash, g_str_equal);
-  for (guint i = 0; i < as_component_box_len (components); i++)
-    {
-      AsComponent *component = NULL;
-      const char  *id        = NULL;
-
-      component = as_component_box_index (components, i);
-      id        = as_component_get_id (component);
+      if (component == NULL)
+        {
+          SEND_AND_RETURN_ERROR (
+              self, TRUE,
+              BZ_FLATPAK_ERROR_APPSTREAM_FAILURE,
+              "Failed to parse appstream component from appstream bundle silo "
+              "originating from download at path %s for remote '%s': %s",
+              appstream_xml_path,
+              remote_name,
+              local_error->message);
+        }
 
+      id = as_component_get_id (component);
       g_hash_table_replace (component_hash, (gpointer) id, component);
     }
 
@@ -1295,6 +1345,148 @@ retrieve_refs_for_remote_fiber (RetrieveRefsForRemoteData *data)
   return dex_future_new_true ();
 }
 
+static DexFuture *
+retrieve_refs_for_noenumerable_remote (RetrieveRefsForRemoteData *data,
+                                       const char                *remote_name,
+                                       FlatpakInstallation       *installation,
+                                       FlatpakRemote             *remote)
+{
+  g_autoptr (BzFlatpakInstance) self   = NULL;
+  GCancellable *cancellable            = data->parent->cancellable;
+  g_autoptr (GError) local_error       = NULL;
+  g_autoptr (GPtrArray) installed_apps = NULL;
+  guint matched                        = 0;
+
+  bz_weak_get_or_return_reject (self, data->parent->self);
+
+  installed_apps = flatpak_installation_list_installed_refs_by_kind (
+      installation,
+      FLATPAK_REF_KIND_APP,
+      cancellable,
+      &local_error);
+
+  if (installed_apps == NULL)
+    SEND_AND_RETURN_ERROR (
+        self, TRUE,
+        BZ_FLATPAK_ERROR_LOCAL_SYNCHRONIZATION_FAILURE,
+        "Failed to enumerate installed apps for non-enumerable remote '%s': %s",
+        remote_name,
+        local_error->message);
+
+  g_debug ("Found %u total installed apps, filtering for remote '%s'",
+           installed_apps->len, remote_name);
+
+  for (guint i = 0; i < installed_apps->len; i++)
+    {
+      FlatpakInstalledRef *iref         = NULL;
+      const char          *ref_origin   = NULL;
+      g_autoptr (AsComponent) component = NULL;
+      g_autoptr (BzFlatpakEntry) entry  = NULL;
+      g_autoptr (GBytes) appstream_gz   = NULL;
+
+      iref       = g_ptr_array_index (installed_apps, i);
+      ref_origin = flatpak_installed_ref_get_origin (iref);
+
+      if (g_strcmp0 (ref_origin, remote_name) != 0)
+        continue;
+
+      matched++;
+
+      appstream_gz = flatpak_installed_ref_load_appdata (iref, cancellable, NULL);
+      if (appstream_gz != NULL)
+        {
+          g_autoptr (GBytes) appstream       = NULL;
+          g_autoptr (XbBuilderSource) source = NULL;
+          g_autoptr (XbSilo) silo            = NULL;
+          g_autoptr (GError) appstream_error = NULL;
+
+          appstream = decompress_appstream_gz (appstream_gz, cancellable, &appstream_error);
+          if (appstream == NULL)
+            {
+              g_info ("Could not decompress appstream for installed ref: %s",
+                      appstream_error ? appstream_error->message : "unknown error");
+              goto create_entry;
+            }
+
+          source = xb_builder_source_new ();
+          if (!xb_builder_source_load_bytes (source, appstream,
+                                             XB_BUILDER_SOURCE_FLAG_LITERAL_TEXT,
+                                             &appstream_error))
+            {
+              g_info ("Could not load appstream bytes: %s",
+                      appstream_error ? appstream_error->message : "unknown error");
+              goto create_entry;
+            }
+
+          silo = build_silo (source, cancellable, &appstream_error);
+          if (silo == NULL)
+            {
+              g_info ("Could not build silo from appstream: %s",
+                      appstream_error ? appstream_error->message : "unknown error");
+              goto create_entry;
+            }
+
+          component = extract_first_component_for_silo (silo, &appstream_error);
+          if (component == NULL)
+            {
+              g_info ("Could not parse appstream component: %s",
+                      appstream_error ? appstream_error->message : "unknown error");
+            }
+        }
+
+    create_entry:
+      entry = bz_flatpak_entry_new_for_ref (
+          FLATPAK_REF (iref),
+          remote,
+          installation == self->user,
+          component,
+          NULL,
+          NULL);
+
+      if (entry != NULL)
+        {
+          g_autoptr (BzBackendNotification) notif = NULL;
+
+          notif = bz_backend_notification_new ();
+          bz_backend_notification_set_kind (notif, BZ_BACKEND_NOTIFICATION_KIND_REPLACE_ENTRY);
+          bz_backend_notification_set_entry (notif, BZ_ENTRY (entry));
+
+          send_notif_all (self, notif, TRUE);
+        }
+    }
+
+  g_debug ("Found %u installed apps from non-enumerable remote '%s'", matched, remote_name);
+
+  {
+    g_autoptr (BzBackendNotification) notif = NULL;
+
+    notif = bz_backend_notification_new ();
+    bz_backend_notification_set_kind (notif, BZ_BACKEND_NOTIFICATION_KIND_TELL_INCOMING);
+    bz_backend_notification_set_n_incoming (notif, matched);
+
+    send_notif_all (self, notif, TRUE);
+  }
+
+  return dex_future_new_true ();
+}
+
+static DexFuture *
+retrieve_refs_for_remote_fiber (RetrieveRefsForRemoteData *data)
+{
+  FlatpakInstallation *installation   = data->installation;
+  FlatpakRemote       *remote         = data->remote;
+  const char          *remote_name    = NULL;
+  gboolean             is_noenumerate = FALSE;
+
+  remote_name    = flatpak_remote_get_name (remote);
+  is_noenumerate = flatpak_remote_get_noenumerate (remote);
+
+  if (is_noenumerate)
+    return retrieve_refs_for_noenumerable_remote (data, remote_name, installation, remote);
+  else
+    return retrieve_refs_for_enumerable_remote (data, remote_name, installation, remote);
+}
+
 static DexFuture *
 retrieve_installs_fiber (GatherRefsData *data)
 {
@@ -1431,6 +1623,81 @@ retrieve_updates_fiber (GatherRefsData *data)
       G_TYPE_PTR_ARRAY, g_steal_pointer (&ids));
 }
 
+static DexFuture *
+list_repositories_fiber (ListReposData *data)
+{
+  g_autoptr (BzFlatpakInstance) self = NULL;
+  GCancellable *cancellable          = NULL;
+  g_autoptr (GError) local_error     = NULL;
+  g_autoptr (GPtrArray) system_repos = NULL;
+  g_autoptr (GPtrArray) user_repos   = NULL;
+  g_autoptr (GListStore) repos       = NULL;
+
+  cancellable = data->cancellable;
+
+  bz_weak_get_or_return_reject (self, data->self);
+
+  repos = g_list_store_new (BZ_TYPE_REPOSITORY);
+
+  if (self->system != NULL)
+    {
+      system_repos = flatpak_installation_list_remotes (
+          self->system, cancellable, &local_error);
+      if (system_repos == NULL)
+        SEND_AND_RETURN_ERROR (
+            self, TRUE,
+            BZ_FLATPAK_ERROR_CANNOT_INITIALIZE,
+            "Failed to enumerate remotes for system installation: %s",
+            local_error->message);
+
+      for (guint i = 0; i < system_repos->len; i++)
+        {
+          FlatpakRemote *remote         = NULL;
+          g_autoptr (BzRepository) repo = NULL;
+
+          remote = g_ptr_array_index (system_repos, i);
+          repo   = g_object_new (BZ_TYPE_REPOSITORY,
+                                 "name", flatpak_remote_get_name (remote),
+                                 "title", flatpak_remote_get_title (remote),
+                                 "url", flatpak_remote_get_url (remote),
+                                 "is-user", FALSE,
+                                 NULL);
+
+          g_list_store_append (repos, repo);
+        }
+    }
+
+  if (self->user != NULL)
+    {
+      user_repos = flatpak_installation_list_remotes (
+          self->user, cancellable, &local_error);
+      if (user_repos == NULL)
+        SEND_AND_RETURN_ERROR (
+            self, TRUE,
+            BZ_FLATPAK_ERROR_CANNOT_INITIALIZE,
+            "Failed to enumerate remotes for user installation: %s",
+            local_error->message);
+
+      for (guint i = 0; i < user_repos->len; i++)
+        {
+          FlatpakRemote *remote         = NULL;
+          g_autoptr (BzRepository) repo = NULL;
+
+          remote = g_ptr_array_index (user_repos, i);
+          repo   = g_object_new (BZ_TYPE_REPOSITORY,
+                                 "name", flatpak_remote_get_name (remote),
+                                 "title", flatpak_remote_get_title (remote),
+                                 "url", flatpak_remote_get_url (remote),
+                                 "is-user", TRUE,
+                                 NULL);
+
+          g_list_store_append (repos, repo);
+        }
+    }
+
+  return dex_future_new_for_object (g_steal_pointer (&repos));
+}
+
 static DexFuture *
 transaction_fiber (TransactionData *data)
 {
@@ -1694,10 +1961,11 @@ transaction_fiber (TransactionData *data)
                  (DexFuture *const *) jobs->pdata,
                  jobs->len),
              NULL);
-  dex_await (dex_future_allv (
-                 (DexFuture *const *) data->send_futures->pdata,
-                 data->send_futures->len),
-             NULL);
+  if (data->send_futures->len > 0)
+    dex_await (dex_future_allv (
+                   (DexFuture *const *) data->send_futures->pdata,
+                   data->send_futures->len),
+               NULL);
 
   errored = g_hash_table_new_full (
       g_direct_hash, g_direct_equal,
@@ -1741,7 +2009,7 @@ transaction_job_fiber (TransactionJobData *data)
     return dex_future_new_reject (
         BZ_FLATPAK_ERROR,
         BZ_FLATPAK_ERROR_TRANSACTION_FAILURE,
-        "Failed to run flatpak transaction on user installation: %s",
+        "Failed to run flatpak transaction on installation: %s",
         local_error->message);
 
   return dex_future_new_true ();
@@ -2194,3 +2462,98 @@ cmp_rref (FlatpakRemoteRef *a,
 
   return 0;
 }
+
+static AsComponent *
+parse_component_for_node (XbNode  *node,
+                          GError **error)
+{
+  g_autofree char *component_xml  = NULL;
+  g_autoptr (AsMetadata) metadata = NULL;
+  AsComponent *component          = NULL;
+  gboolean     result             = FALSE;
+
+  component_xml = xb_node_export (node, XB_NODE_EXPORT_FLAG_NONE, error);
+  if (component_xml == NULL)
+    return NULL;
+
+  metadata = as_metadata_new ();
+  result   = as_metadata_parse_data (
+      metadata,
+      component_xml,
+      -1,
+      AS_FORMAT_KIND_XML,
+      error);
+  if (!result)
+    return NULL;
+
+  component = as_metadata_get_component (metadata);
+  return bz_object_maybe_ref (component);
+}
+
+static GBytes *
+decompress_appstream_gz (GBytes       *appstream_gz,
+                         GCancellable *cancellable,
+                         GError      **error)
+{
+  g_autoptr (GZlibDecompressor) decompressor = NULL;
+  g_autoptr (GInputStream) stream_gz         = NULL;
+  g_autoptr (GInputStream) stream_data       = NULL;
+  g_autoptr (GBytes) appstream               = NULL;
+
+  decompressor = g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP);
+  stream_gz    = g_memory_input_stream_new_from_bytes (appstream_gz);
+  stream_data  = g_converter_input_stream_new (stream_gz, G_CONVERTER (decompressor));
+
+  appstream = g_input_stream_read_bytes (
+      stream_data,
+      0x100000, /* 1MB */
+      cancellable,
+      error);
+  if (appstream == NULL)
+    return NULL;
+
+  return g_steal_pointer (&appstream);
+}
+
+static XbSilo *
+build_silo (XbBuilderSource *source,
+            GCancellable    *cancellable,
+            GError         **error)
+{
+  g_autoptr (XbBuilder) builder = NULL;
+  const gchar *const *locales   = NULL;
+  g_autoptr (XbSilo) silo       = NULL;
+
+  builder = xb_builder_new ();
+
+  locales = g_get_language_names ();
+  for (guint i = 0; locales[i] != NULL; i++)
+    xb_builder_add_locale (builder, locales[i]);
+
+  xb_builder_import_source (builder, source);
+  silo = xb_builder_compile (
+      builder,
+      XB_BUILDER_COMPILE_FLAG_NATIVE_LANGS,
+      cancellable,
+      error);
+
+  return g_steal_pointer (&silo);
+}
+
+static AsComponent *
+extract_first_component_for_silo (XbSilo  *silo,
+                                  GError **error)
+{
+  g_autoptr (XbNode) root        = NULL;
+  g_autoptr (GPtrArray) children = NULL;
+
+  root     = xb_silo_get_root (silo);
+  children = xb_node_get_children (root);
+
+  if (children == NULL || children->len == 0)
+    return NULL;
+
+  return parse_component_for_node (
+      g_ptr_array_index (children, 0),
+      error);
+}
