@@ -51,6 +51,7 @@
 #include "bz-state-info.h"
 #include "bz-stats-dialog.h"
 #include "bz-tag-list.h"
+#include "bz-template-callbacks.h"
 
 struct _BzFullView
 {
@@ -196,79 +197,6 @@ bz_full_view_set_property (GObject      *object,
     }
 }
 
-static gboolean
-invert_boolean (gpointer object,
-                gboolean value)
-{
-  return !value;
-}
-
-static gboolean
-is_zero (gpointer object,
-         int      value)
-{
-  return value == 0;
-}
-
-static gboolean
-is_positive (gpointer object,
-             int      value)
-{
-  return value > -1;
-}
-
-static gboolean
-is_longer (gpointer    object,
-           GListModel *model,
-           int         value)
-{
-  if (model == NULL)
-    return FALSE;
-  return g_list_model_get_n_items (model) > value;
-}
-
-static gboolean
-is_null (gpointer object,
-         GObject *value)
-{
-  return value == NULL;
-}
-
-static gboolean
-is_empty (gpointer    object,
-          GListModel *model)
-{
-  if (model == NULL)
-    return TRUE;
-  return g_list_model_get_n_items (model) == 0;
-}
-
-static gboolean
-is_empty_string (gpointer    object,
-                 const char *str)
-{
-  if (str == NULL)
-    return TRUE;
-  return *str == '\0';
-}
-
-static gboolean
-logical_and (gpointer object,
-             gboolean value1,
-             gboolean value2)
-{
-  return value1 && value2;
-}
-
-static char *
-bool_to_string (gpointer object,
-                gboolean condition,
-                char    *if_true,
-                char    *if_false)
-{
-  return g_strdup (condition ? if_true : if_false);
-}
-
 static char *
 format_with_small_suffix (char *number, const char *suffix)
 {
@@ -339,6 +267,9 @@ format_size (gpointer object, guint64 value)
   char            *decimal  = NULL;
   int              digits   = 0;
 
+  if (value == 0)
+    return g_strdup (_ ("N/A"));
+
   if (space != NULL)
     {
       *space = '\0';
@@ -356,12 +287,12 @@ format_size (gpointer object, guint64 value)
   return g_strdup (size_str);
 }
 
-static  char *
+static char *
 get_size_label (gpointer object,
                 gboolean is_installable)
 {
   // Translators: .
-  return g_strdup (is_installable ? _("Download") : _("Installed"));
+  return g_strdup (is_installable ? _ ("Download") : _ ("Installed"));
 }
 
 static guint64
@@ -378,7 +309,12 @@ get_size_type (gpointer object,
 static char *
 format_size_tooltip (gpointer object, guint64 value)
 {
-  g_autofree char *size_str = g_format_size (value);
+  g_autofree char *size_str = NULL;
+
+  if (value == 0)
+    return g_strdup (_ ("Size information unavailable"));
+
+  size_str = g_format_size (value);
   return g_strdup_printf (_ ("Download size of %s"), size_str);
 }
 
@@ -673,7 +609,7 @@ get_safety_rating_label (gpointer object,
   switch (importance)
     {
     case BZ_IMPORTANCE_UNIMPORTANT:
-      return g_strdup (_ ("Low Risk"));
+      return g_strdup (_ ("Safe"));
     case BZ_IMPORTANCE_NEUTRAL:
       return g_strdup (_ ("Low Risk"));
     case BZ_IMPORTANCE_INFORMATION:
@@ -1249,6 +1185,7 @@ bz_full_view_class_init (BzFullViewClass *klass)
   g_type_ensure (BZ_TYPE_ENTRY_GROUP);
   g_type_ensure (BZ_TYPE_FADING_CLAMP);
   g_type_ensure (BZ_TYPE_FAVORITE_BUTTON);
+  g_type_ensure (BZ_TYPE_FLATPAK_ENTRY);
   g_type_ensure (BZ_TYPE_HARDWARE_SUPPORT_DIALOG);
   g_type_ensure (BZ_TYPE_LAZY_ASYNC_TEXTURE_MODEL);
   g_type_ensure (BZ_TYPE_SECTION_VIEW);
@@ -1259,6 +1196,8 @@ bz_full_view_class_init (BzFullViewClass *klass)
   g_type_ensure (BZ_TYPE_CONTEXT_TILE);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/io/github/kolunmi/Bazaar/bz-full-view.ui");
+  bz_widget_class_bind_all_util_callbacks (widget_class);
+
   gtk_widget_class_bind_template_child (widget_class, BzFullView, stack);
   gtk_widget_class_bind_template_child (widget_class, BzFullView, main_scroll);
   gtk_widget_class_bind_template_child (widget_class, BzFullView, shadow_overlay);
@@ -1267,15 +1206,6 @@ bz_full_view_class_init (BzFullViewClass *klass)
   gtk_widget_class_bind_template_child (widget_class, BzFullView, wide_install_button);
   gtk_widget_class_bind_template_child (widget_class, BzFullView, narrow_install_button);
   gtk_widget_class_bind_template_child (widget_class, BzFullView, narrow_open_button);
-  gtk_widget_class_bind_template_callback (widget_class, invert_boolean);
-  gtk_widget_class_bind_template_callback (widget_class, is_zero);
-  gtk_widget_class_bind_template_callback (widget_class, is_positive);
-  gtk_widget_class_bind_template_callback (widget_class, is_null);
-  gtk_widget_class_bind_template_callback (widget_class, is_empty);
-  gtk_widget_class_bind_template_callback (widget_class, is_empty_string);
-  gtk_widget_class_bind_template_callback (widget_class, logical_and);
-  gtk_widget_class_bind_template_callback (widget_class, is_longer);
-  gtk_widget_class_bind_template_callback (widget_class, bool_to_string);
   gtk_widget_class_bind_template_callback (widget_class, format_favorites_count);
   gtk_widget_class_bind_template_callback (widget_class, format_recent_downloads);
   gtk_widget_class_bind_template_callback (widget_class, format_recent_downloads_tooltip);
@@ -1378,13 +1308,29 @@ bz_full_view_set_entry_group (BzFullView   *self,
 
   if (group != NULL)
     {
-      g_autoptr (DexFuture) future = NULL;
-
       self->group    = g_object_ref (group);
       self->ui_entry = bz_entry_group_dup_ui_entry (group);
 
-      future            = bz_entry_group_dup_all_into_store (group);
-      self->group_model = bz_result_new (future);
+      if (self->ui_entry != NULL && bz_result_get_resolved (self->ui_entry))
+        {
+          g_autoptr (GListStore) store = NULL;
+          g_autoptr (DexFuture) future = NULL;
+          BzEntry *entry               = NULL;
+
+          entry = bz_result_get_object (self->ui_entry);
+          store = g_list_store_new (BZ_TYPE_ENTRY);
+          g_list_store_append (store, entry);
+
+          future            = dex_future_new_for_object (store);
+          self->group_model = bz_result_new (future);
+        }
+      else
+        {
+          g_autoptr (DexFuture) future = NULL;
+
+          future            = bz_entry_group_dup_all_into_store (group);
+          self->group_model = bz_result_new (future);
+        }
 
       adw_view_stack_set_visible_child_name (self->stack, "content");
     }
