@@ -36,8 +36,8 @@
 #include "bz-installed-page.h"
 #include "bz-io.h"
 #include "bz-progress-bar.h"
-#include "bz-result.h"
 #include "bz-search-widget.h"
+#include "bz-template-callbacks.h"
 #include "bz-transaction-dialog.h"
 #include "bz-transaction-list-dialog.h"
 #include "bz-transaction-manager.h"
@@ -202,43 +202,6 @@ bz_window_set_property (GObject      *object,
     }
 }
 
-static gboolean
-invert_boolean (gpointer object,
-                gboolean value)
-{
-  return !value;
-}
-
-static gboolean
-is_double_zero (gpointer object,
-                double   value)
-{
-  return value == 0.0;
-}
-
-static gboolean
-is_null (gpointer object,
-         GObject *value)
-{
-  return value == NULL;
-}
-
-static gboolean
-logical_and (gpointer object,
-             gboolean value1,
-             gboolean value2)
-{
-  return value1 && value2;
-}
-
-static gboolean
-logical_or (gpointer object,
-            gboolean value1,
-            gboolean value2)
-{
-  return value1 || value2;
-}
-
 static char *
 list_length (gpointer    object,
              GListModel *model)
@@ -584,6 +547,8 @@ bz_window_class_init (BzWindowClass *klass)
   g_type_ensure (BZ_TYPE_FLATHUB_PAGE);
 
   gtk_widget_class_set_template_from_resource (widget_class, "/io/github/kolunmi/Bazaar/bz-window.ui");
+  bz_widget_class_bind_all_util_callbacks (widget_class);
+
   gtk_widget_class_bind_template_child (widget_class, BzWindow, comet_overlay);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, split_view);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, transactions_stack);
@@ -601,11 +566,6 @@ bz_window_class_init (BzWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, BzWindow, main_view_stack);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, main_stack);
   gtk_widget_class_bind_template_child (widget_class, BzWindow, debug_id_label);
-  gtk_widget_class_bind_template_callback (widget_class, invert_boolean);
-  gtk_widget_class_bind_template_callback (widget_class, is_double_zero);
-  gtk_widget_class_bind_template_callback (widget_class, is_null);
-  gtk_widget_class_bind_template_callback (widget_class, logical_and);
-  gtk_widget_class_bind_template_callback (widget_class, logical_or);
   gtk_widget_class_bind_template_callback (widget_class, list_length);
   gtk_widget_class_bind_template_callback (widget_class, browser_group_selected_cb);
   gtk_widget_class_bind_template_callback (widget_class, search_widget_select_cb);
@@ -763,6 +723,19 @@ transact_fiber (TransactData *data)
   else
     id_dup = g_strdup (bz_entry_get_id (data->entry));
 
+  /* Prevent Bazaar from being removed by itself */
+  if (data->remove)
+    {
+      const char *bazaar_id = NULL;
+
+      bazaar_id = g_application_get_application_id (g_application_get_default ());
+      if (g_strcmp0 (id_dup, bazaar_id) == 0)
+        {
+          bz_show_error_for_widget (GTK_WIDGET (self), _ ("You can't remove Bazaar from Bazaar!"));
+          return dex_future_new_false ();
+        }
+    }
+
   // Show the dialog
   dialog_result = dex_await_pointer (
       bz_transaction_dialog_show (
@@ -912,13 +885,19 @@ void
 bz_window_show_entry (BzWindow *self,
                       BzEntry  *entry)
 {
-  /* TODO: IMPLEMENT ME! */
-  bz_show_error_for_widget (
-      GTK_WIDGET (self),
-      _ ("The ability to inspect and install local .flatpak bundle files is coming soon! "
-         "In the meantime, try running\n\n"
-         "flatpak install --bundle your-bundle.flatpak\n\n"
-         "on the command line."));
+  g_autoptr (BzEntryGroup) group  = NULL;
+  AdwNavigationPage *visible_page = NULL;
+
+  g_return_if_fail (BZ_IS_WINDOW (self));
+  g_return_if_fail (BZ_IS_ENTRY (entry));
+
+  group = bz_entry_group_new_for_single_entry (entry);
+
+  bz_full_view_set_entry_group (self->full_view, group);
+
+  visible_page = adw_navigation_view_get_visible_page (self->navigation_view);
+  if (visible_page != adw_navigation_view_find_page (self->navigation_view, "view"))
+    adw_navigation_view_push_by_tag (self->navigation_view, "view");
 }
 
 void
